var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { createSerialOperation } from '@deep-foundation/deeplinks/imports/gql/index.js';
import { getAllDeviceInfo } from './get-all-device-info.js';
/**
 * Saves device information by modifying the value of the device link.
 * Any fields not included in the `info` parameter will not be modified.
 * If the {@link SaveDeviceInfoParam.info} parameter is undefined, the function will call {@link getAllDeviceInfo} to get the device information.
 *
 * @example
 * #### Save all device information
```ts

import { saveDeviceInfo, getAllDeviceInfo } from '@deep-foundation/capacitor-device';

// Implicitly (if you do not pass info to saveDeviceInfo - it will save all information by default)
await saveDeviceInfo({deep, deviceLinkId});

// Explicitly
const allDeviceInfo = await getAllDeviceInfo();
await saveDeviceInfo({deep, deviceLinkId, info: allDeviceInfo});
```
 * #### Save specific device information
```ts
import { saveDeviceInfo } from '@deep-foundation/capacitor-device';

const deviceBatteryInfo = await Device.getBatteryInfo();
await saveDeviceInfo({deep, deviceLinkId, info: deviceBatteryInfo});

const {value: languageCode} = await Device.getLanguageCode();
await saveDeviceInfo({deep, deviceLinkId, info: {languageCode}});

const {value: languageTag} = await Device.getLanguageTag();
await saveDeviceInfo({deep, deviceLinkId, info: {languageTag}});
```
#### Save information manually
```ts
const myDeviceData = {
  "identifier": "d787dc6b-041f-4a61-8078-d5573c2e7a4e",
  "model": "X11",
  "platform": "web",
  "isVirtual": false,
  "osVersion": "x86.64",
  "isCharging": true,
  "languageTag": "en",
  "batteryLevel": 1,
  "languageCode": "en",
  "manufacturer": "Google Inc.",
  "webViewVersion": "113.0.0.0",
  "operatingSystem": "unknown"
};

await saveDeviceInfo({deep, deviceLinkId, info: myDeviceData});
```
 *
 */
export function saveDeviceInfo(param) {
    return __awaiter(this, void 0, void 0, function* () {
        const { deep, info: data } = param;
        const deviceLink = yield getDeviceLink();
        const value = yield getValue({
            deviceLink,
            data,
        });
        const serialOperations = yield getSerialOperations({
            deviceLink,
            value,
        });
        yield deep.serial({
            operations: serialOperations,
        });
        function getDeviceLink() {
            return __awaiter(this, void 0, void 0, function* () {
                let deviceLink;
                if ('deviceLinkId' in param) {
                    if (!param.deviceLinkId) {
                        throw new Error(`deviceLinkId is undefined`);
                    }
                    const { data } = yield deep.select({
                        id: param.deviceLinkId,
                    });
                    deviceLink = data[0];
                }
                else if ('deviceLink' in param) {
                    if (!param.deviceLink) {
                        throw new Error(`deviceLink is undefined`);
                    }
                    deviceLink = param.deviceLink;
                }
                else {
                    throw new Error(`Either deviceLink or deviceLinkId must be passed`);
                }
                return deviceLink;
            });
        }
        function getValueInsertSerialOperation({ deviceLink, value, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return createSerialOperation({
                    table: 'objects',
                    type: 'insert',
                    objects: {
                        link_id: deviceLink.id,
                        value: value,
                    },
                });
            });
        }
        function getValueUpdateSerialOperation({ deviceLink, value, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return createSerialOperation({
                    table: 'objects',
                    type: 'update',
                    exp: {
                        link_id: deviceLink.id,
                    },
                    value: {
                        value: value,
                    },
                });
            });
        }
        function getValue({ deviceLink, data, }) {
            var _a, _b;
            return __awaiter(this, void 0, void 0, function* () {
                return Object.assign(Object.assign({}, ((_b = (_a = deviceLink.value) === null || _a === void 0 ? void 0 : _a.value) !== null && _b !== void 0 ? _b : {})), (data !== null && data !== void 0 ? data : (yield getAllDeviceInfo())));
            });
        }
        function getSerialOperations({ deviceLink, value, }) {
            return __awaiter(this, void 0, void 0, function* () {
                let serialOperations = [];
                if (!deviceLink.value) {
                    serialOperations.push(yield getValueUpdateSerialOperation({
                        deviceLink,
                        value,
                    }));
                }
                else {
                    serialOperations.push(yield getValueInsertSerialOperation({
                        deviceLink,
                        value,
                    }));
                }
                return serialOperations;
            });
        }
    });
}
